package com.fluentcommerce.model.common;

import com.fluentretail.api.model.attribute.Attribute;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class Log {

    public String title;
    public String message;
    public List<Attribute> attributes;

    public Log() {
        attributes = new ArrayList<Attribute>();
    }

    public static Log create(String title, String message, List<Attribute> attributes) {
        Objects.requireNonNull(title);
        Objects.requireNonNull(message);

        Log result = new Log();

        result.title = title;
        result.message = message;

        if (attributes != null) {
            result.attributes = attributes;
        }

        return result;
    }

    public static Log create(String title, String message) {
        return create(title, message, null);
    }

    public Log addAttribute(Attribute attribute) {
        Objects.requireNonNull(attribute);

        attributes.add(attribute);
        return this;
    }

    public Log addAttribute(String name, Attribute.Type type, Object value) {
        Attribute attribute = Attribute.builder().name(name).attributeType(type).value(value).build();

        return addAttribute(attribute);
    }
}
