package com.fluentretail.se.plugins.util;

import com.fluentcommerce.common.ContextWrapper;
import com.fluentcommerce.model.common.LogCollection;
import com.fluentretail.api.model.attribute.Attribute;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

@Slf4j
public class CommonUtils {

    public static void newLog(ContextWrapper context) {
        LogCollection logs = context.getLogCollection();
        if (logs.isEmpty()) {
            return;
        }

        String ruleName = context.getRuleName();

        Objects.requireNonNull(context);
        Guard.requireNotNullString(ruleName);
        Objects.requireNonNull(logs);

        String serverDate = getServerTimeAsText();

        Attribute dateAttribute = Attribute.builder()
            .name("Date")
            .attributeType(Attribute.Type.STRING)
            .value(serverDate)
            .build();

        Attribute logCollectionAttribute = Attribute.builder()
            .name("LogCollection")
            .attributeType(Attribute.Type.OBJECT)
            .value(logs)
            .build();

        List<Attribute> attributes = new ArrayList<Attribute>() {
            {
                add(dateAttribute);
                add(logCollectionAttribute);
            }
        };

        if (context.getEvent() != null) {
            Attribute eventAttribute = Attribute.builder()
                .name("EventAttribute")
                .attributeType(Attribute.Type.OBJECT)
                .value(context.getEvent().getAttributes())
                .build();
            attributes.add(eventAttribute);
        }

        if (Thread.currentThread() != null) {
            Attribute threadNameAttribute = Attribute.builder()
                .name("ThreadName")
                .attributeType(Attribute.Type.STRING)
                .value(Thread.currentThread().getName())
                .build();
            attributes.add(threadNameAttribute);
        }

        if (CommonUtils.class.getClassLoader() != null) {
            Attribute classLoaderNameAttribute = Attribute.builder()
                .name("ClassLoader")
                .attributeType(Attribute.Type.STRING)
                .value(CommonUtils.class.getClassLoader().toString())
                .build();
            attributes.add(classLoaderNameAttribute);
        }

        context.action().log(ruleName, serverDate, attributes);
    }

    public static String getServerTimeAsText() {
        Date serverDateNow = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/YYYY HH:mm z", Locale.getDefault());
        return dateFormat.format(serverDateNow);
    }

    public static boolean isEmptyMap(Map<?, ?> map) {
        return map == null || map.isEmpty();
    }

    public static boolean isEmptyOrBlank(String value) {
        return StringUtils.isEmpty(value) || StringUtils.isBlank(value);
    }
}