package com.fluentretail.se.plugins.util.helpers;

import com.fluentcommerce.jetpack.rubix.utils.rulelogger.logger.RuleLogger;

public class LogHelper {

    private final Level currentLevel;
    private final RuleLogger logger;

    public LogHelper(Level currentLevel, RuleLogger logger) {
        this.currentLevel = currentLevel;
        this.logger = logger;
    }

    public LogHelper(String levelName, RuleLogger logger) {
        this(getLevelByName(levelName), logger);
    }

    public static Level getLevelByName(final String name) {
        Level level;
        try {
            level = Level.valueOf(name);
        } catch (IllegalArgumentException | NullPointerException e) {
            level = Level.INFO;
        }
        return level;
    }

    public void debug(String message) {
        if (Level.DEBUG.isLevelEnabled(currentLevel)) {
            //use info space for debug message, because RuleLogger does not have debug space
            logger.info(message);
        }

    }

    public void info(String message) {
        if (Level.INFO.isLevelEnabled(currentLevel)) {
            logger.info(message);
        }

    }

    public void warn(String message) {
        if (Level.WARN.isLevelEnabled(currentLevel)) {
            logger.warn(message);
        }

    }

    public void error(String message) {
        if (Level.ERROR.isLevelEnabled(currentLevel)) {
            logger.error(message);
        }

    }

    public enum Level {
        DEBUG(40),
        INFO(30),
        WARN(20),
        ERROR(10),
        OFF(0);

        private final int code;

        Level(final int code) {
            this.code = code;
        }

        public boolean isLevelEnabled(final Level level) {
            return code <= level.code;
        }
    }
}
